#! /bin/bash

# Script to check that the BOOST libraries are linkable to
# a program compiled with the GMP compilation flags.
# We check just libboost_filesystem.a, and if that works OK
# then we assume all the BOOST libs are fine too.

if [ $# -ne 3 ]
then
  echo "$0: expecting 3 args (CXX, CXXFLAGS, BOOST_LIBS)" > /dev/stderr
  exit 1
fi

CXX="$1"
CXXFLAGS="$2"
BOOST_LIBS="$3"

umask 22

# We create a temp dir and work in there; at the end we
# just delete the whole dir -- that should clean up properly.

TMP_DIR=boost-check-arch-$HOSTNAME-$UID-$$
/bin/rm -rf $TMP_DIR  &&  mkdir $TMP_DIR 2>/dev/null
if [ $? -ne 0 ]
then
  echo "$0: Unable to create temporary working directory ($TMP_DIR)"
  exit 2
fi

cd $TMP_DIR

PROG_NAME=test
# Here is the simple source code we shall use to test the compiler:
cat > $PROG_NAME.C <<EOF
#include "boost/filesystem.hpp"
using namespace boost::filesystem;

int main()
{
  path file = "$PROG_NAME.C";
  if (!exists(file) || !is_regular_file(file)) exit(2);
}
EOF


#$CXX $PROG_NAME.C -o $PROG_NAME $CXXFLAGS $BOOST_LIB_DIR/libboost_filesystem.a $BOOST_LIB_DIR/libboost_system.a  2> /dev/null
$CXX $PROG_NAME.C -o $PROG_NAME $CXXFLAGS $BOOST_LIBS  2> /dev/null
if [ $? -ne 0 -o \! -f $PROG_NAME -o \! -x $PROG_NAME ]
then
  echo "compilation failed"
  cd ..; /bin/rm -rf $TMP_DIR
  echo "Compilation of CoCoALib's BOOST test prog failed"
  exit 3
fi

./$PROG_NAME  2> /dev/null
if [ $? -ne 0 ]
then
  echo "???"
  cd ..; /bin/rm -rf $TMP_DIR
  echo "BOOST lib test prog gave run-time error"
  exit 4
fi

cd ..; /bin/rm -rf $TMP_DIR
