#!/bin/bash

# This script looks for the necessary BOOST sub-libraries.
# If found, it prints out two SHELL assignments (to be processed
# using the eval commmand) to BOOST_LIB_DIR & BOOST_LIBS.
# If not, it prints out a warning and returns with exit code 1.

SUBLIBS="thread filesystem system"

##################################################################
if [ "$#" != "1" -o -z "$1" ]
then
  echo "$0: ERROR: requires 1 non-empty arg (full path of BOOST_HDR_DIR)"
  exit 1
fi

BOOST_HDR_DIR="$1"

# Special handling if BOOST is not fully installed...
if [ -d "$BOOST_HDR_DIR/stage" -a -r "$BOOST_HDR_DIR/stage" -a -d "$BOOST_HDR_DIR/stage/lib" -a -r "$BOOST_HDR_DIR/stage/lib" ]
then
  BOOST_LIB_DIR="$BOOST_HDR_DIR/stage/lib"
else
  # CHEAP HACK: prefer lib64 over lib if possible -- GMP probably chose 64-bits
  BOOST_LIB_DIR_DIR="`dirname $BOOST_HDR_DIR`"
  if [ -d "$BOOST_LIB_DIR_DIR/lib64" ]
  then
    BOOST_LIB_DIR="$BOOST_LIB_DIR_DIR/lib64"
  else
    BOOST_LIB_DIR="$BOOST_LIB_DIR_DIR/lib"
  fi
fi

# Check that the SUBLIBS we need are present.
# Check first for multi-threaded versions...
MISSING_SUBLIBS_MT=
BOOST_LIBS_MT=
for sublib in $SUBLIBS
do
  /bin/ls "$BOOST_LIB_DIR"/libboost_$sublib-mt* >/dev/null 2>&1
  if [ $? = 0 ]
  then
    BOOST_LIBS_MT="$BOOST_LIBS_MT -lboost_$sublib-mt"
  else
    MISSING_SUBLIBS_MT="$MISSING_SUBLIBS_MT libboost_$sublib-mt"
  fi
done
    
if [ -z "$MISSING_SUBLIBS_MT" ]
then
  echo "BOOST_LIB_DIR=\"$BOOST_LIB_DIR\"; BOOST_LIBS=\"$BOOST_LIBS_MT\""
  exit 0
fi

# Check now for single-threaded versions...
MISSING_SUBLIBS=
BOOST_LIBS=
for sublib in $SUBLIBS
do
  /bin/ls "$BOOST_LIB_DIR"/libboost_$sublib* >/dev/null 2>&1
  if [ $? = 0 ]
  then
    BOOST_LIBS="$BOOST_LIBS -lboost_$sublib"
  else
    MISSING_SUBLIBS="$MISSING_SUBLIBS libboost_$sublib"
  fi
done
    
if [ -z "$MISSING_SUBLIBS" ]
then
  echo "BOOST_LIB_DIR=\"$BOOST_LIB_DIR\"; BOOST_LIBS=\"$BOOST_LIBS\""
  exit 0
fi

# Neither single- nor multi-threaded libs had all we need, so give error.
if [ -z "$BOOST_LIBS_MT" ]
then
  echo "REASON: CoCoA-5 needs the following missing BOOST sublibraries: $MISSING_SUBLIBS"
else
  echo "REASON: CoCoA-5 needs the following missing BOOST sublibraries: $MISSING_SUBLIBS_MT"
fi
exit 1
