#! /bin/sh

if [ "$#" != "1" ]
then
  echo "ERROR: $0 requires 1 arg (full path of gmp.h)" > /dev/stderr
  exit 1
fi

# Assume the file exists and is readable -- already checked in configure script.
GMP_H="$1"
GMP_INC_DIR=`dirname "$GMP_H"`

# Below we create a small C++ program for printing out the GMP compilation flags.
# We use two temporary files.
TMPFILE=get-gmp-cflags
/bin/rm -rf $TMPFILE $TMPFILE.C
cat > $TMPFILE.C <<EOF
#include "gmp.h"
#include <iostream>

int main()
{
  std::cout << __GMP_CFLAGS << std::endl;
}
EOF

$CXX -I "$GMP_INC_DIR" $TMPFILE.C -o $TMPFILE 2> /dev/null
# if [ $? -ne 0 ]
# then
#   /bin/rm -f $TMPFILE.C $TMPFILE
#   exit 1
# fi
GMP_CXXFLAGS=`./$TMPFILE`
/bin/rm -rf $TMPFILE $TMPFILE.C


# GMP_CXXFLAGS contains all the compilation flags used when building GMP.
# We pick out just the compilation options which begin with -m
CoCoALib_CXXFLAGS=
for opt in $GMP_CXXFLAGS
do
    case $opt in
	( -m* )
	CoCoALib_CXXFLAGS="$CoCoALib_CXXFLAGS $opt";;
    esac
done

echo $CoCoALib_CXXFLAGS
