//   Copyright (c) 2012-2014 Anna M. Bigatti, Christof Soeger
//
//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "BuiltInFunctions.H"
#include "BuiltInOneLiners.H"

using namespace std;
using namespace boost;
using namespace boost::iostreams;
using namespace CoCoA::AST;
using namespace CoCoA::LexerNS;
using namespace CoCoA::ParserNS;

namespace CoCoA {
namespace InterpreterNS {

//  extern std::vector<NameFunPair> builtIns; // declared in BuiltInFunctions.C

#ifdef CoCoA_WITH_NORMALIZ


// DECLARE_STD_BUILTIN_FUNCTION(NmzHilbertBasis, 1) {
// // 	intrusive_ptr<ListValue> listOfList = runtimeEnv->evalArgAs<ListValue>(ARG(0));
// //   vector<vector<BigInt> > v;
// // 	const ListValue::ContainerType::size_type nLists = listOfList->size();
// // 	for (ListValue::ContainerType::size_type a=0; a<nLists; ++a) {
// // 		intrusive_ptr<ListValue> l = dynamic_pointer_cast<ListValue>(listOfList->getValue(a));
// // 		if (!l)
// // 			throw RuntimeException("The argument is not a list of lists", ARG(0).exp);
// //     v.push_back(vector<BigInt>());
// // 		const ListValue::ContainerType::size_type len = l->size();
// // 		for (ListValue::ContainerType::size_type b=0; b<len; ++b)
// //     {
// //       intrusive_ptr<BigIntValue> n = dynamic_pointer_cast<BigIntValue>(l->getValue(b));
// //       if (!n)
// //         throw RuntimeException("The argument is not a list of lists on INT", ARG(0).exp);
// // 			v[a].push_back(n->theInteger);
// //     }
// // 	}
// 	intrusive_ptr<MatrixValue> M = runtimeEnv->evalArgAs<MatrixValue>(ARG(0));
//   return Value::from(NmzHilbertBasis_forC5(M->theMatrix));
// }
// END_STD_BUILTIN_FUNCTION


// ---- Nmz OneLiners ----

DECLARE_COCOALIBFORC5_FUNCTION1(NmzHilbertBasis, MatrixValue)

// ---- Nmz STD_BUILTIN_FUNCTION ----

DECLARE_STD_BUILTIN_FUNCTION(NmzNormalToricRing, 1) {
  vector<RingElem> v = runtimeEnv->evalArgAsRingElemList(ARG(0));
  return Value::from(NmzNormalToricRing_forC5(v));
}
END_STD_BUILTIN_FUNCTION

DECLARE_STD_BUILTIN_FUNCTION(NmzIntClosureToricRing, 1) {
  vector<RingElem> v = runtimeEnv->evalArgAsRingElemList(ARG(0));
  return Value::from(NmzIntClosureToricRing_forC5(v));
}
END_STD_BUILTIN_FUNCTION

DECLARE_STD_BUILTIN_FUNCTION(NmzIntClosureMonIdeal, 1) {
  vector<RingElem> v = runtimeEnv->evalArgAsRingElemList(ARG(0));
  return Value::from(NmzIntClosureMonIdeal_forC5(v));
}
END_STD_BUILTIN_FUNCTION

// ---- Nmz BUILTIN_FUNCTION (+ variable number of args) ----

/*
DECLARE_STD_BUILTIN_FUNCTION(NmzComputation, 1) {
  return NmzComputation(ARG(0),libnormaliz::ConeProperty::DefaultMode);
}
END_STD_BUILTIN_FUNCTION
*/

DECLARE_ARITYCHECK_FUNCTION(NmzComputation) { return (1<=nArg) && (nArg<=2); }
DECLARE_BUILTIN_FUNCTION(NmzComputation) {
  invocationExpression->checkNumberOfArgs(1,2);// variable number of args
  intrusive_ptr<RecordValue> cone = runtimeEnv->evalArgAs<RecordValue>(ARG(0));
  vector<string> fieldnames = cone->myFieldNamesStrings();

  intrusive_ptr<RecordValue> out(new RecordValue);
  map< libnormaliz::InputType, vector<vector<BigInt> > > InputMap;
  try {
  for (long i=0; i<len(fieldnames); ++i)
  {
    intrusive_ptr<RightValue> x = boost::dynamic_pointer_cast<RightValue>(cone->getField(fieldnames[i]));
    if (intrusive_ptr<MatrixValue> M = dynamic_pointer_cast<MatrixValue>(x)) {
      // transform M to a vector< vector<BigInt> >
      // the following line may throw a libnormaliz::BadInputException
      InputMap[Normaliz::ToInputType(fieldnames[i])] = Normaliz::MatrixToVecVecBigInt(M->theMatrix);
    } //TODO allow also [[..],..,[..]] input?
    else
      throw WrongTypeException(MatrixValue::type->name
//                             RingElemValue::type->name + " or " +
                               + " as field \"" + fieldnames[i] + "\" of the record",
                               x->getType()->name, ARG(0).exp);
  }

  Normaliz::cone C(InputMap);

  // collect what to compute
  libnormaliz::ConeProperties CPs;
  if (invocationExpression->args.size()==1)
  {
    CPs.set(libnormaliz::ConeProperty::DefaultMode);
  }
  else
  {
    vector<string> vs = runtimeEnv->evalArgAsListOf<StringValue>(ARG(1));
    for (long i=0; i<len(vs); ++i)
    {
      // invalid strings will cause a libnormaliz::BadInputException
      CPs.set(libnormaliz::toConeProperty(vs[i]));
    }
  }

  C.myComputation(CPs);

  // fill the return record with everthing we can
  for (size_t i=0; i<libnormaliz::ConeProperty::EnumSize; i++) {
    libnormaliz::ConeProperty::Enum prop_i = static_cast<libnormaliz::ConeProperty::Enum>(i);
    if (C.isComputed(prop_i)) switch (prop_i) {
      case libnormaliz::ConeProperty::HilbertBasis:
        out->setField(libnormaliz::toString(prop_i), Value::from(HilbertBasis(C)));
        break;
      case libnormaliz::ConeProperty::ModuleGenerators:
        out->setField(libnormaliz::toString(prop_i), Value::from(ModuleGenerators(C)));
        break;
      case libnormaliz::ConeProperty::Generators:
        out->setField(libnormaliz::toString(prop_i), Value::from(Generators(C)));
        break;
      case libnormaliz::ConeProperty::ExtremeRays:
        out->setField(libnormaliz::toString(prop_i), Value::from(ExtremeRays(C)));
        out->setField("Rank", Value::from(rank(C)));
        break;
      case libnormaliz::ConeProperty::VerticesOfPolyhedron:
        out->setField(libnormaliz::toString(prop_i), Value::from(VerticesOfPolyhedron(C)));
        break;
      case libnormaliz::ConeProperty::Deg1Elements:
        out->setField(libnormaliz::toString(prop_i), Value::from(Deg1Elements(C)));
        break;
      case libnormaliz::ConeProperty::GeneratorsOfToricRing:
        out->setField(libnormaliz::toString(prop_i), Value::from(GeneratorsOfToricRing(C)));
        break;
      case libnormaliz::ConeProperty::SupportHyperplanes:
        out->setField(libnormaliz::toString(prop_i), Value::from(SupportHyperplanes(C)));
        out->setField("Equations", Value::from(Equations(C)));
        out->setField("Congruences", Value::from(Congruences(C)));
        break;
      case libnormaliz::ConeProperty::ExcludedFaces:
        out->setField(libnormaliz::toString(prop_i), Value::from(ExcludedFaces(C)));
        break;
      case libnormaliz::ConeProperty::HilbertSeries:
        out->setField(libnormaliz::toString(prop_i), Value::from(HilbertSeries(C)));
        // out->setField("HilbertPolynomial", Value::from(HilbertPoly(C))); //only if periode is 1
        break;
      case libnormaliz::ConeProperty::Multiplicity:
        out->setField(libnormaliz::toString(prop_i), Value::from(multiplicity(C)));
        break;
      case libnormaliz::ConeProperty::Grading:
        out->setField(libnormaliz::toString(prop_i), Value::from(grading(C)));
        break;
      case libnormaliz::ConeProperty::IsDeg1HilbertBasis:
        out->setField(libnormaliz::toString(prop_i), Value::from(IsDeg1HilbertBasis(C)));
        break;
      case libnormaliz::ConeProperty::IsPointed:
        out->setField(libnormaliz::toString(prop_i), Value::from(IsPointed(C)));
        break;
      case libnormaliz::ConeProperty::IsIntegrallyClosed:
        out->setField(libnormaliz::toString(prop_i), Value::from(IsIntegrallyClosed(C)));
        break;
      case libnormaliz::ConeProperty::RecessionRank:
        out->setField(libnormaliz::toString(prop_i), Value::from(RecessionRank(C)));
        break;
      case libnormaliz::ConeProperty::AffineDim:
        out->setField(libnormaliz::toString(prop_i), Value::from(AffineDim(C)));
        break;
      case libnormaliz::ConeProperty::ModuleRank:
        out->setField(libnormaliz::toString(prop_i), Value::from(ModuleRank(C)));
        break;
      case libnormaliz::ConeProperty::Shift:
        out->setField(libnormaliz::toString(prop_i), Value::from(shift(C)));
        break;
      case libnormaliz::ConeProperty::Dehomogenization:
        out->setField(libnormaliz::toString(prop_i), Value::from(dehomogenization(C)));
        break;
      default:
        break;
    }
  }

  out->setField("EmbeddingDim", Value::from(EmbeddingDim(C)));
  out->setField("IsInhomogeneous", Value::from(IsInhomogeneous(C)));

  } catch (const libnormaliz::NormalizException& e) {
    CoCoA_ERROR(e.what(),"NmzComputation");
  }

  return out;
}

#endif // CoCoA_WITH_NORMALIZ

} // namespace AST
} // namespace CoCoA
