//   Copyright (c) 2010-2012 Giovanni Lagorio and John Abbott, Anna M. Bigatti
//
//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#ifndef BUILTINFUNCTIONS_H_
#define BUILTINFUNCTIONS_H_

#include <iostream>
#include <sstream>
#include <exception>
#include <cassert>
#include <vector>
#include <cstdlib>
#include <memory>
#include <boost/scope_exit.hpp>
#include <boost/foreach.hpp>
#include <boost/algorithm/string/case_conv.hpp>
#include <boost/iostreams/device/file.hpp>
#include <boost/iostreams/filtering_stream.hpp>
#include <boost/iostreams/filter/newline.hpp>
#if 0
#include <zip.h>
#endif

#include "CoCoA/library.H"
#include "CoCoALibSupplement.H"
#include "Interpreter.H"
#include "CompilationDate.H"
#include "OnlineHelp.H"
#include "VersionInfo.H"

// using namespace std;
// using namespace boost;
// using namespace boost::iostreams;
// using namespace CoCoA::AST;
// using namespace CoCoA::LexerNS;
// using namespace CoCoA::ParserNS;

namespace CoCoA {
namespace InterpreterNS {

template <> BigInt inline theValue<BigIntValue>(boost::intrusive_ptr<BigIntValue> x) { return x->theInteger; }
template <> const BigInt inline theValue<const BigIntValue>(boost::intrusive_ptr<const BigIntValue> x) { return x->theInteger; }
template <> BigRat inline theValue<BigRatValue>(boost::intrusive_ptr<BigRatValue> x) { return x->theRational; }
template <> const BigRat inline theValue<const BigRatValue>(boost::intrusive_ptr<const BigRatValue> x) { return x->theRational; }
template <> RingElem inline theValue<RingElemValue>(boost::intrusive_ptr<RingElemValue> x) { return x->theRingElem; }
template <> const RingElem inline theValue<const RingElemValue>(boost::intrusive_ptr<const RingElemValue> x) { return x->theRingElem; }
template <> ring inline theValue<RingValue>(boost::intrusive_ptr<RingValue> x) { return x->theRing; }
template <> const ring inline theValue<const RingValue>(boost::intrusive_ptr<const RingValue> x) { return x->theRing; }
template <> RingHom inline theValue<RingHomValue>(boost::intrusive_ptr<RingHomValue> x) { return x->theRingHom; }
template <> const RingHom inline theValue<const RingHomValue>(boost::intrusive_ptr<const RingHomValue> x) { return x->theRingHom; }
template <> matrix inline theValue<MatrixValue>(boost::intrusive_ptr<MatrixValue> x) { return x->theMatrix; }
template <> const matrix inline theValue<const MatrixValue>(boost::intrusive_ptr<const MatrixValue> x) { return x->theMatrix; }
template <> std::string inline theValue<StringValue>(boost::intrusive_ptr<StringValue> x) { return x->theString; }
template <> const std::string inline theValue<const StringValue>(boost::intrusive_ptr<const StringValue> x) { return x->theString; }
template <> std::string inline theValue<TypeValue>(boost::intrusive_ptr<TypeValue> x) { return x->name; }
template <> const std::string inline theValue<const TypeValue>(boost::intrusive_ptr<const TypeValue> x) { return x->name; }
template <> bool inline theValue<BoolValue>(boost::intrusive_ptr<BoolValue> x) { return x->theBool; }
template <> bool inline theValue<const BoolValue>(boost::intrusive_ptr<const BoolValue> x) { return x->theBool; }
template <> ideal inline theValue<IdealValue>(boost::intrusive_ptr<IdealValue> x) { return x->theIdeal; }
template <> const ideal inline theValue<const IdealValue>(boost::intrusive_ptr<const IdealValue> x) { return x->theIdeal; }
template <> module inline theValue<ModuleValue>(boost::intrusive_ptr<ModuleValue> x) { return x->theModule; }
template <> const module inline theValue<const ModuleValue>(boost::intrusive_ptr<const ModuleValue> x) { return x->theModule; }
template <> ModuleElem inline theValue<ModuleElemValue>(boost::intrusive_ptr<ModuleElemValue> x) { return x->theModuleElem; }
template <> const ModuleElem inline theValue<const ModuleElemValue>(boost::intrusive_ptr<const ModuleElemValue> x) { return x->theModuleElem; }

inline const std::string& PtrCastString(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const StringValue>(x)->theString; }
inline const BigInt& PtrCastBigInt(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const BigIntValue>(x)->theInteger; }
inline const BigRat& PtrCastBigRat(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const BigRatValue>(x)->theRational; }
inline const RingElem& PtrCastRingElem(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const RingElemValue>(x)->theRingElem; }
inline const ModuleElem& PtrCastModuleElem(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const ModuleElemValue>(x)->theModuleElem; }
inline const ring& PtrCastRing(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const RingValue>(x)->theRing; }
inline const module& PtrCastModule(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const ModuleValue>(x)->theModule; }
inline const ideal& PtrCastIdeal(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const IdealValue>(x)->theIdeal; }
inline const matrix& PtrCastMatrix(boost::intrusive_ptr<const RightValue> x)
{ return intrusive_ptr_cast<const MatrixValue>(x)->theMatrix; }

  //------------------------------------------------------------
  // The fn  builtIns  behaves like a global variable;  it is filled
  // by the ctor for AddBuiltIn, and the filled vector is used in the
  // the fn initBuiltInFunctions -- see BuiltInFunctions.C

typedef std::pair<std::string, boost::intrusive_ptr<BuiltInFunction> > NameFunPair;

std::vector<NameFunPair>& builtIns(); // declared in BuiltInFunctions.C

struct AddBuiltIn
{
  inline AddBuiltIn(const std::string &fname, boost::intrusive_ptr<BuiltInFunction> fun)
  {
    builtIns().push_back(std::make_pair(fname, fun));
  }
};

  //------------------------------------------------------------

#define ARG(N) invocationExpression->args[N]

#define DECLARE_ARITYCHECK_FUNCTION(FNAME) bool builtin_check_arity_##FNAME(int nArg)
#define DECLARE_BUILTIN_FUNCTION(FNAME) \
		intrusive_ptr<RightValue> builtin_##FNAME(const intrusive_ptr<const InvocationExpression> invocationExpression, RuntimeEnvironment * const runtimeEnv); \
		DECLARE_ARITYCHECK_FUNCTION(FNAME) ;\
		AddBuiltIn add_##FNAME(#FNAME, new BuiltInFunction(builtin_##FNAME, builtin_check_arity_##FNAME)); \
		intrusive_ptr<RightValue> builtin_##FNAME(const intrusive_ptr<const InvocationExpression> invocationExpression, RuntimeEnvironment * const runtimeEnv)
#define DECLARE_STD_BUILTIN_FUNCTION(FNAME, NARG) \
	DECLARE_ARITYCHECK_FUNCTION(FNAME) { return nArg==(NARG); } \
	DECLARE_BUILTIN_FUNCTION(FNAME) { \
	invocationExpression->checkNumberOfArgs(NARG);
#define END_STD_BUILTIN_FUNCTION }

} // namespace AST
} // namespace CoCoA

#endif /* BUILTINFUNCTIONS_H_ */
