//   Copyright (c) 2012-2013 Giovanni Lagorio. John Abbott, Anna M. Bigatti
//
//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "BuiltInOneLiners.H"

using namespace std;
using namespace boost;
using namespace boost::iostreams;
using namespace CoCoA::AST;
using namespace CoCoA::LexerNS;
using namespace CoCoA::ParserNS;

namespace CoCoA {
namespace InterpreterNS {

//  extern std::vector<NameFunPair> builtIns; // declared in BuiltInFunctions.C

////// functions on BIGINT
DECLARE_COCOALIB_FUNCTION1(factorial, BigIntValue)
DECLARE_COCOALIB_FUNCTION1(fibonacci, BigIntValue)
DECLARE_COCOALIB_FUNCTION1(isqrt, BigIntValue) // AMB
DECLARE_COCOALIB_FUNCTION1(NextProbPrime, BigIntValue) // AMB
DECLARE_COCOALIB_FUNCTION1(PrevProbPrime, BigIntValue) // JAA
DECLARE_COCOALIB_FUNCTION2(valuation, BigIntValue, BigIntValue) // JAA
DECLARE_COCOALIB_FUNCTION2(RoundDiv, BigIntValue, BigIntValue) // JAA
DECLARE_COCOALIB_FUNCTION2(iroot, BigIntValue, BigIntValue) // AMB
DECLARE_COCOALIB_FUNCTION3(PowerMod, BigIntValue, BigIntValue, BigIntValue)
DECLARE_COCOALIB_FUNCTION1(IsEven, BigIntValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsOdd, BigIntValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsPrime, BigIntValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsProbPrime, BigIntValue) // AMB
DECLARE_COCOALIB_FUNCTION1(PrimitiveRoot, BigIntValue) // JAA
DECLARE_COCOALIBFORC5_FUNCTION1(NextPrime, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION2(random, BigIntValue, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION1(ContFrac, BigRatValue)
DECLARE_COCOALIBFORC5_FUNCTION1(CFApproximants, BigRatValue)
DECLARE_COCOALIBFORC5_FUNCTION2(BinomialRepr, BigIntValue, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION4(BinomialReprShift, BigIntValue, BigIntValue, BigIntValue, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION1(NumPartitions, BigIntValue)

////// functions on BIGRAT
DECLARE_COCOALIB_FUNCTION2(CFApprox, BigRatValue, BigRatValue) // AMB

////// functions on RINGELEM
DECLARE_COCOALIB_FUNCTION1(CanonicalRepr, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION1(ClearDenom, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION1(CommonDenom, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION1(ContentFreeFactor, RingElemValue)
DECLARE_COCOALIB_FUNCTION1(IsConstant, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsIndet, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsIndetPosPower, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsInvertible, RingElemValue)
DECLARE_COCOALIB_FUNCTION1(IsMonomial, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsPthPower, RingElemValue) // JAA
DECLARE_COCOALIB_FUNCTION1(IsZeroDivisor, RingElemValue) // AMB
// DECLARE_COCOALIB_FUNCTION1(LF, RingElemValue) // AMB -- also for ideals
DECLARE_COCOALIB_FUNCTION1(PthRoot, RingElemValue) // JAA
DECLARE_COCOALIB_FUNCTION1(content, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION1(factor, RingElemValue)
DECLARE_COCOALIB_FUNCTION1(SqFreeFactor, RingElemValue)
DECLARE_COCOALIB_FUNCTION1(monic, RingElemValue) // AMB //if LC(f)==1???
DECLARE_COCOALIB_FUNCTION2(IsDivisible, RingElemValue, RingElemValue)
DECLARE_COCOALIB_FUNCTION2(deriv, RingElemValue, RingElemValue) // AMB
DECLARE_COCOALIB_FUNCTION2(power, RingElemValue, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION1(IsTerm, RingElemValue)
DECLARE_COCOALIBFORC5_FUNCTION1(monomials, RingElemValue)
DECLARE_COCOALIBFORC5_FUNCTION1(support, RingElemValue)
DECLARE_COCOALIBFORC5_FUNCTION2(CoeffListWRT, RingElemValue, RingElemValue) // AMB+JAA
DECLARE_COCOALIBFORC5_FUNCTION2(CoeffOfTerm, RingElemValue, RingElemValue)
DECLARE_COCOALIBFORC5_FUNCTION2(DensePoly, RingValue, BigIntValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION2(SmoothFactor, BigIntValue, BigIntValue)

////// functions on MODULEELEM
DECLARE_COCOALIB_FUNCTION1(FirstNonZero, ModuleElemValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION1(FirstNonZeroPosn, ModuleElemValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION1(LPosn, ModuleElemValue) // AMB

////// functions on RING (and PolyRing)
DECLARE_COCOALIB_FUNCTION0(RingQQ)
DECLARE_COCOALIB_FUNCTION0(RingZZ)
DECLARE_COCOALIB_FUNCTION1(characteristic, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsCommutative, RingValue)
// DECLARE_COCOALIB_FUNCTION1(IsStdGraded, RingValue) /// should be here
DECLARE_COCOALIB_FUNCTION1(IsField, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsFiniteField, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsFractionField, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsIntegralDomain, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsOrderedDomain, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsPolyRing, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsQQ, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsQuotientRing, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsTrueGCDDomain, RingValue)
DECLARE_COCOALIB_FUNCTION1(IsZZ, RingValue)
DECLARE_COCOALIB_FUNCTION1(LogCardinality, RingValue)
DECLARE_COCOALIB_FUNCTION1(NewFractionField, RingValue)  // AMB
DECLARE_COCOALIB_FUNCTION1(NewRingFp, BigIntValue)  // AMB
DECLARE_COCOALIB_FUNCTION1(one, RingValue)
DECLARE_COCOALIB_FUNCTION1(owner, RingElemValue)
//DECLARE_COCOALIB_FUNCTION1(zero, RingValue)  // now also for modules
//DECLARE_COCOALIB_FUNCTION1(NewWeylAlgebra, ..)  // AMB
DECLARE_COCOALIB_FUNCTION2(NewQuotientRing, RingValue, IdealValue)  // AMB
DECLARE_COCOALIBFORC5_FUNCTION1(GradingDim, RingValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION1(RingQQt, BigIntValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION1(WeightsMatrix, RingValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION1(OrdMat, RingValue) // AMB

////// functions for RINGHOM
DECLARE_COCOALIB_FUNCTION1(IdentityHom, RingValue) // AMB
DECLARE_COCOALIB_FUNCTION1(codomain, RingHomValue) // AMB
DECLARE_COCOALIB_FUNCTION1(domain, RingHomValue) // AMB
DECLARE_COCOALIB_FUNCTION2(CanonicalHom, RingValue, RingValue) // GL
DECLARE_COCOALIB_FUNCTION2(TmpChainCanonicalHom, RingValue, RingValue) // AMB

////// functions on MATRIX
//DECLARE_COCOALIB_FUNCTION1(FrobeniusNorm2, MatrixValue) // AMB
//DECLARE_COCOALIB_FUNCTION1(OperatorNorm1, MatrixValue) // AMB
//DECLARE_COCOALIB_FUNCTION1(OperatorNormInfinity, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsAntiSymmetric, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsDiagonal, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsSymmetric, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsTermOrdering, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(LinKer, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(NumCols, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(NumRows, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(PseudoInverse, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(adjoint, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(det, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(inverse, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION1(rank, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION2(ConcatAntiDiag, MatrixValue, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION2(LinSolve, MatrixValue, MatrixValue) // JAA
DECLARE_COCOALIB_FUNCTION2(ConcatDiag, MatrixValue, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION2(ConcatHor, MatrixValue, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION2(ConcatVer, MatrixValue, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION2(TensorMat, MatrixValue, MatrixValue) // AMB
DECLARE_COCOALIB_FUNCTION4(BlockMat2x2, MatrixValue, MatrixValue, MatrixValue, MatrixValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION1(ExtractOrdMat, MatrixValue)
DECLARE_COCOALIBFORC5_FUNCTION1(LexMat, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION1(RevLexMat, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION1(StdDegLexMat, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION1(StdDegRevLexMat, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION1(XelMat, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION1(transposed, MatrixValue)
DECLARE_COCOALIBFORC5_FUNCTION2(GetCol, MatrixValue, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION2(GetRow, MatrixValue, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION2(IdentityMat, RingValue, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION3(NewDenseMat, RingValue, BigIntValue, BigIntValue)
DECLARE_COCOALIBFORC5_FUNCTION3(ZeroMat, RingValue, BigIntValue, BigIntValue)
//DECLARE_COCOALIBFORC5_FUNCTION4(FilledMat, RingValue, BigIntValue, BigIntValue, RingElemValue) // removed from CoCoALib
DECLARE_COCOALIBFORC5_FUNCTION3(ClosePassingPoly, RingValue, MatrixValue, MatrixValue)

////// functions on IDEAL
DECLARE_COCOALIB_FUNCTION1(AlexanderDual, IdealValue) // AMB
DECLARE_COCOALIB_FUNCTION1(AmbientRing, IdealValue) // AMB
DECLARE_COCOALIB_FUNCTION1(AreGensMonomial, IdealValue) // AMB
DECLARE_COCOALIB_FUNCTION1(HilbertSeriesQuot, IdealValue) // AMB
DECLARE_COCOALIB_FUNCTION1(IsZeroDim, IdealValue) // AMB
DECLARE_COCOALIB_FUNCTION1(JanetBasis, IdealValue) // AMB
DECLARE_COCOALIB_FUNCTION1(PrimaryDecomposition, IdealValue) // AMB
//DECLARE_COCOALIB_FUNCTION1(GBasis, IdealValue) now also for modules
//DECLARE_COCOALIB_FUNCTION1(MinGens, IdealValue) now also for modules
//DECLARE_COCOALIB_FUNCTION1(gens, IdealValue)   now also for modules
//DECLARE_COCOALIB_FUNCTION2(IsContained, IdealValue, IdealValue) now also for modules
//DECLARE_COCOALIB_FUNCTION2(IsElem, RingElemValue, IdealValue) now also for modules
DECLARE_COCOALIB_FUNCTION2(NF, RingElemValue, IdealValue) // AMB
DECLARE_COCOALIB_FUNCTION2(colon, IdealValue, IdealValue) // AMB
DECLARE_COCOALIB_FUNCTION2(intersect, IdealValue, IdealValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION1(HilbertNumQuot, IdealValue) // AMB

////// functions on MODULE
DECLARE_COCOALIBFORC5_FUNCTION2(SubmoduleCols, ModuleValue, MatrixValue) // AMB
DECLARE_COCOALIBFORC5_FUNCTION2(SubmoduleRows, ModuleValue, MatrixValue) // AMB

////// UTILS
DECLARE_COCOALIBFORC5_FUNCTION0(CpuTime)
DECLARE_COCOALIBFORC5_FUNCTION0(date)
DECLARE_COCOALIBFORC5_FUNCTION0(TimeOfDay)  // time is a keyword (unused)
// DECLARE_COCOALIB_FUNCTION0(NewSymbol)
// DECLARE_COCOALIB_FUNCTION1(NewSymbols, BigIntValue)

} // namespace AST
} // namespace CoCoA
